# File tres/Scripts/ctres.cl
# July 2, 2009
# By Doug Mink, SAO Telescope Data Center

# CTRES - Process multiple similar TRES images from raw data files to spectra

procedure ctres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir=""	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="no"	{prompt="Process combined 2-D file (sum mean median only(sum) no)"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	extract=yes	{prompt="Extract orders to echelle file (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
string  refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
int	plotorder=23	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot
int	pl, n
bool	vb, dt, sr, sv, disp, rc, sum, sumonly, delim, iscomp
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	rc = recenter
	pl = plotorder
	idir = instdir
	ddir = datedir
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse
	delim = delimage
	ct = threshold

	if (ims == "") {
	    print ("CTRES: Reduce multiple similar TRES images, removing cosmic rays")
	    print ("Usage: ctres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles.adddate = no
	trsfiles.keeppath = no
	trsfiles.extension = ".fits"
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("CTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.overscan = yes
	tproc.subbias = no
	tproc.subdark = no
	tproc.flatten = flatten
	tproc.cosmic = yes
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.extract = extract
	tproc.disperse = disperse
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.sumfile = ""
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = rc
	tproc.refmode = refmode
	tproc.medshift = medshift
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims)

	if (delim) {
	    imdelete (ims)
	    if (tproc.sumfile != "")
		imdelete (tproc.sumfile)
	    }

# Plot extracted spectra if desired
	if (pl != 0) {
	    list = tempout
	    while (fscan (list,imout) != EOF) {
		fileroot (imout)
		imroot = fileroot.rootname
		smatch (imroot, "COMP")
		iscomp = smatch.found
		ims = imroot // "_f1.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f2.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f3.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		}
	    if (tproc.sumfile != "") {
		fileroot (tproc.sumfile)
		imroot = fileroot.rootname
		smatch (imroot, "COMP")
		iscomp = smatch.found
		ims = imroot // "_f1.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f2.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f3.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		}
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)

end

# Jan 17 2008	New task for quick look with cosmic ray removal
# Jan 25 2008	Access data by sequence number as well as file name
# Jan 25 2008	Add wavelength calibration and plotting
# Jan 30 2008	Use tdisp to add wavelength solution
# Jan 30 2008	Add parameters apdir and aproot for extraction templates
# Feb 11 2008	Change temporary file names from ctresin and ctresout
# Feb 12 2008	Use TPROC to process spectra
# Feb 13 2008	Change parameter imdate to datedir
# Feb 15 2008	Use trsfiles to parse input image file list
# Mar 26 2008	Change saveraw and saveold default to no, plotorder to 23
# Apr  4 2008	Add sumspec parameter to process sum of images
# Apr 22 2008	Save TCOSMIC bad pixel, median, and limit files if saveold
# Apr 29 2008	Add delimage to allow deletion of processed image files
# Apr 29 2008	Set trsfiles.overwrite=no to keep already-processed files
# Apr 30 3008	Fix bug by passing saveraw parameter to TPROC
# Jul 10 2008	Make extraction optional for flat processing
# Aug 13 2008	Name files _f1 _f2 _f3 instead of _fib1 _fib2 _fib3
# Sep  9 2008	Plot resulting spectra using THARPLOT, not SPLOT
# Sep 19 2008	Add medshift parameter which gets passed through TPROC to TCAL1
# Sep 19 2008	Add refmode parameter which gets passed through TPROC to TDISP1
# Oct 14 2008	Set instdir from tres package parameter
# Oct 14 2008	Make default to save raw files
# Oct 15 2008	Add threshold parameter to set cosmic ray threshold
# Oct 16 2008	From raw directory, switch to quick-look directory and back
# Oct 17 2008	Change default to not delete the processed image
# Dec  5 2008	Make @ optional if image is *.list file

# Mar  9 2009	Return if no files found from list
# Apr 10 2009	Make default normdir = "" (working directory)
# Apr 16 2009	If plotorder > 0, plot object spectra with splot
# Apr 16 2009	If plotorder < 0, plot object spectra with trsplot -plotorder
# Jun  9 2009	Plot summed file if present and plotting
# Jun 10 2009	Delete combined image when processed images are being deleted
# Jul  2 2009	Set overscan and subbias to subtract overscan
# Jul  2 2009	Set all parameters of TRSFILES
