# File tres/Scripts/otres.cl
# July 2, 2009
# By Doug Mink, SAO Telescope Data Center

# OTRES - Process multiple similar TRES images from raw data files to spectra

procedure otres (images)

string	images=""	{prompt="Images to process"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool    overscan=no     {prompt="Subtract overscan (yes or no)"}
bool	subbias=yes	{prompt="Subtract bias frame (yes or no)"}
bool	subdark=yes	{prompt="Subtract darkimage (yes or no)"}
file	darkfile=""	{prompt="Dark file to subtract (compute if null)"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir=""	{prompt="Directory for normalization file"}
bool	cosmic=no	{prompt="Remove cosmic rays (yes or not)"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="sum"	{prompt="Process combined 2-D file (sum mean median only(sum) no)"}
bool	rmscat=no	{prompt="Remove scattered light from image (yes or no)"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	extract=yes	{prompt="Extract orders to echelle file (yes or no)"}
bool	rmblaze=no	{prompt="Remove echelle blaze function (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
string  refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
bool	subsky=no	{prompt="Subtract sky spectrum (yes or no)"}
int	plotorder=23	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath, dfile, flat1, flat2
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot, tdate
int	pl, n
bool	vb, dt, sr, sv, disp, rc, delim, iscompm
bool	rms, rmb, subs
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	rc = recenter
	pl = plotorder
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse
	delim = delimage
	dfile = darkfile
	ct = threshold
	rms = rmscat
	rmb = rmblaze
	subs = subsky

	if (ims == "") {
	    print ("OTRES: Reduce multiple similar TRES images, optionally removing cosmic rays")
	    print ("Usage: otres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }

#  Make list file from input file parameter
	trsfiles.instdir = ctres.instdir
	trsfiles.datedir = ctres.datedir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles.adddate = no
	trsfiles.keeppath = no
	trsfiles.extension = ".fits"
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("OTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	if (dfile == "")
	    tproc.subdark = no
	else
	    tproc.subdark = yes
	tproc.darkfile = dfile
	tproc.flatten = flatten
	tproc.cosmic = cosmic
	tproc.overscan = overscan
	tproc.subbias = subbias
	tproc.subdark = subdark
	tproc.darkfile = darkfile
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.extract = extract
	tproc.disperse = disperse
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.sumfile = ""
	tproc.rmscat = rms
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = rc
	tproc.refmode = refmode
	tproc.medshift = medshift
	tproc.rmblaze = rmb
	tproc.subsky = subs
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims)

	if (delim) {
	    imdelete (ims)
	    if (tproc.sumfile != "")
		imdelete (tproc.sumfile)
	    }

# Plot extracted spectra if desired
	if (pl > 0) {
	    list = tempout
	    while (fscan (list,imout) != EOF) {
		fileroot (imout)
		imroot = fileroot.rootname
		smatch (imroot, "COMP")
		iscomp = smatch.found
		ims = imroot // "_f1.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f2.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f3.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		}
	    if (tproc.sumfile != "") {
		fileroot (tproc.sumfile)
		imroot = fileroot.rootname
		smatch (imroot, "COMP")
		iscomp = smatch.found
		ims = imroot // "_f1.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f2.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		ims = imroot // "_f3.ec.fits"
		if (access (ims)) {
		    if (iscomp)
			tharplot (ims, pl)
		    else if (pl > 0)
			splot (ims, pl)
		    else if (pl < 0)
			trsplot (ims, -pl)
		    }
		}
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)

end

# Apr 15 2009	New task based on ctres with optional cosmic ray removal
# Apr 16 2009	If plotorder > 0, plot object spectra with splot
# Apr 17 2009	If plotorder < 0, plot object spectra with trsplot -plotorder
# Apr 23 2009	Add rmscat to optionally remove scattered light
# Apr 23 2009	Add rmblaze to optionally remove echelle blaze function
# Apr 27 2009	Add darkfile parameter to pass to TPROC -> TPROC1 -> CCDPROC
# Apr 29 2009	Add option to subtract the sky spectrum
# Apr 30 2009	Separate sumspec string parameter into booleans sumspec, sumonly
# May 12 2009	Drop avgspec sumonly parameters; add mean median only options to sumspec
# May 14 2009	Fix dark subtraction
# May 18 2009	Add subdark parameter
# Jun  9 2009	Plot summed file if present and plotting
# Jun 10 2009	Delete combined image file if other image files are deleted
# Jul  2 2009	Set all parameters of TRSFILES
# Jul  2 2009	Add overscan and subbias to subtract bias instead of overscan
