# File tres/Scripts/trsdate.cl
# By Doug Mink
# July 22, 2009

# TRSDATE -- Set instrument and date directories for TRES listing and quick look

procedure trsdate (datedir)

string	datedir="now"	{prompt="Date for images (now=today)"}

string	instdir="/tdata/tres/" {prompt="Root directory with raw data"}
string	quickdir="home$tresq/"	{prompt="Root directory for quicklook data"}
string	rdir=""	{prompt="Raw data directory (returned) (null if no data)"}
string	wdir=""	{prompt="Working directory (returned) (null if no data)"}
int	lastseq=200	{prompt="First sequence number for summed spectra"}
bool	deltemp=yes	{prompt="Delete temporary list file (yes or no)"}
bool	verbose=yes	{prompt="Print table of image parameters (yes or no)"}

struct *dirfiles

begin
    bool vb
    int idate
    file ddir, idir, datadir, qdir
    string newdatadir, ddir0, olddate

    vb = verbose
    ddir = datedir

    fixdir (instdir)
    idir = fixdir.newdir
    if (!access (idir)) {
	print ("TRSDATE: Raw data directory ",idir, " not found")
#	return
	}
    else if (verbose)
	print ("TRSDATE: Raw data directory ",idir, " found")

    fixdir (quickdir)
    qdir = fixdir.newdir

    if (ddir == "now" || ddir == "" || ddir == "0") {
	gettime (0.0, timediff=99)
	ddir = gettime.datadir
	}
    else if (ddir == "now-1" || ddir == "1") {
	gettime (0.0, timediff=-24)
	ddir = gettime.datadir
	}
    else {
	idate = int (ddir)
	if (idate > 9999) {
	    if (idate < 100000) {
		ddir0 = ddir
		if (substr (ddir0,1,1) != "0") {
		    ddir = "200" // substr (ddir0,1,1) // "." // substr (ddir0,2,5)
		    }
		else {
		    ddir = "20" // substr (ddir0,1,2) // "." // substr (ddir0,3,6)
		    }
		}
	    else if (idate < 1000000) {
		ddir = "20" // substr (ddir0,1,2) // "." // substr (ddir0,3,6)
		}
	    else {
		ddir = substr (ddir0,1,4) // "." // substr (ddir0,5,8)
		}
	    }
	}
    if (strlen (ddir) == 8)
	ddir = ddir // "0"

    datadir = idir // ddir
    if (!access (datadir)) {
	newdatadir = datadir // "0"
	if (access (newdatadir)) {
	    datadir = newdatadir
	    ddir = ddir // "0"
	    rdir = datadir
	    }
	else {
	    if (vb){
		print ("TRSDATE: Raw data directory ",datadir, " not found")
		}
#	    return
	    if (rdir != "") {
		smatch (rdir, "/20")
		if (smatch.found)
		    ddir = substr (rdir,smatch.start+1,100)
		else {
		    smatch (wdir, "/20")
		    if (smatch.found)
			ddir = substr (wdir,smatch.start+1,100)
		    else {
			if (verbose)
			    print ("TRSDATE: No past date set")
			return
			}
		    }
		}
	    }
	}
    else
	rdir = datadir
    if (verbose)
	print ("TRSDATE: Raw data directory is ", rdir)

    qtres.instdir = idir
    ctres.datedir = ddir
    ctres.instdir = idir
    trsproc.datedir = ddir
    trsproc.instdir = idir
    ftres.datedir = ddir
    ftres.instdir = idir
    ttres.datedir = ddir
    ttres.instdir = idir
    trslist.datedir = ddir
    trslist.instdir = idir
    trsgroup.datedir = ddir
    trsgroup.instdir = idir
    trsfiles.datedir = ddir
    trsfiles.instdir = idir
    trsfiles.adddate = no
    tcheck.datedir = ddir

    if (!access (qdir)) {
	mkdir (qdir)
	if (!access (qdir)) {
	    print ("TRSDATE: ",qdir, " cannot be created")
	    return
	    }
	}
    wdir = qdir // ddir
    if (!access (wdir)) {
	mkdir (wdir)
	if (!access (wdir)) {
	    print ("TRSDATE: ",wdir, " cannot be created")
	    wdir = ""
	    return
	    }
	}
    cd (wdir)

end

# Feb 13 2008	New program
# Feb 15 2008	Add trsfiles
# Apr  4 2008	First sequence number for summed spectra on this night
# Apr  4 2008	Move to local date directory, creating it if necessary
# Apr  4 2008	Set trsfiles.adddate=no because date is not needed in name
# Apr 22 2008	Add quicklook primary directory and create it if needed
# May 16 2008	Add input time argument to gettime() 
# Jul 14 2008	Add TRSGROUP
# Jul 16 2008	Add TRSPROC, FTRES, and TTRES
# Jul 31 2008	Set next sequence number for sums and averages in this directory
# Aug  1 2008	Skip files which start with "flat" or "comp"
# Aug 14 2008	List files correctly and delete temporary list file
# Oct 14 2008	Print input and output directories on separate lines
# Oct 14 2008	Change default raw data directory to /tdata/tres/
# Nov 12 2008	Add trailing zero to date string if necessary
# Dec 15 2008	Add timediff argument to GETTIME to get current time

# Feb  6 2009	Add trailing zero to date if IRAF has dropped it
# Feb 19 2009	Print message about not finding data directory only in verbose mode
# Feb 19 2009	Return full pathname of working directory in wdir parameter
# Feb 19 2009	Return full pathname of raw data directory in rdir parameter
# Feb 19 2009	Initialize directories in new BTRES task
# Mar  5 2009	Use input datedir parameter only once in task
# Mar  6 2009	Drop all sequence number updating code; it is now in tnext
# Apr  6 2009	Continue if raw data directory is not found instead of quitting
# Apr 27 2009	Drop setting of BTRES directories; that task reads them from CTRES
# Jul 22 2009	Do not create directory if no data
