#*** File Src/gettres.x
#*** March 6, 2009
#*** By Doug Mink

#  GETTRES -- Opens a FITS or IRAF TRES echelle spectrum and returns the
#  image file descriptors and extracted header information.
#  Adapted from the rvsao/Util/getimage.x
 
include	<imhdr.h>
include	<smw.h>
 
define	LEN_USER_AREA	100000
 
procedure gettres (image,mspec,pix,im,sh,nord,npix,name,mode,debug)
 
char	image[ARB]	# Spectrum image file name
int	mspec		# Number of spectrum to read from multispec or 2-D file
pointer	pix		# Spectrum pointer [returned]
pointer	im		# Image header structure [returned]
pointer	sh		# Spectrum header structure [returned]
int	nord		# Number of orders in spectrum [returned]
int	npix		# Number of pixels in spectrum [returned]
char	name[ARB]	# Name of object [returned]
int	mode		# Mode (READ_ONLY, READ_WRITE)
bool	debug
 
pointer mw, smw_openim()
int	nline, ap, ndim
int	nband		# Band number is 1 for TRES
int	ip
int	ltitle
char	temp[16]
# char	units[32]
char	impath[SZ_FNAME]
char	imname[SZ_FNAME]

int	imaccf(), imaccess()
int	strlen(), strldxs(), strncmp()
pointer	immap()
errchk	immap()
 
begin
	im = NULL
	pix = NULL
	sh = NULL
	nord = 0
	npix = 0
	nband = 1
	nline = mspec
	call strcpy (image, impath, SZ_FNAME)
	call strcpy (image, imname, SZ_FNAME)
	if (debug) {
	    call printf ("GETTRES: image file %s about to be read\n")
		call pargstr (imname)
	    call flush (STDOUT)
	    }

	if (imaccess (impath, mode) == NO) {
	    if (imaccess (impath, READ_ONLY) == NO) {
		if (debug) {
		    call printf ("GETTRES: image file %s cannot be read\n")
			call pargstr (imname)
		    call flush (STDOUT)
		    }
		im = ERR
		return
		}
	    else {
		call eprintf ("GETTRES: Cannot write to image %s\n")
		    call pargstr (image)
		}
	    }

#  Map the image
	iferr (im = immap (imname, mode, LEN_USER_AREA)) {
	    if (im != NULL) call imunmap (im)
	    if (debug) {
		call printf ("GETTRES: image file %s cannot be opened\n")
		    call pargstr (imname)
		call flush (STDOUT)
		}
	    iferr (im = immap (image, mode, LEN_USER_AREA)) {
		if (mode == READ_ONLY)
		    call eprintf ("GETTRES: Cannot read image %s\n")
		else
		    call eprintf ("GETTRES: Cannot write to image %s\n")
		    call pargstr (image)
		im = ERR
		return
		}
	    else {
		if (mode == READ_ONLY)
		    call eprintf ("GETTRES: Cannot read image %s\n")
		else
		    call eprintf ("GETTRES: Cannot write to image %s\n")
		    call pargstr (image)
		im = ERR
		return
		}
	    }

# Print image dimensions and return if not 2
	ndim = IM_NDIM(im)
	if (ndim == 2) {
	    if (debug) {
		call printf ("GETTRES: %d x %d %d-D image: spectrum %d\n")
		    call pargi (IM_LEN(im,1))
		    call pargi (IM_LEN(im,2))
		    call pargi (ndim)
		    call pargi (nline)
		call flush (STDOUT)
		}
	    }
	else {
	    if (debug) {
		call printf ("GETTRES: %d x %d x %d %d-D image: spectrum %d\n")
		    call pargi (IM_LEN(im,1))
		    call pargi (IM_LEN(im,2))
		    call pargi (IM_LEN(im,3))
		    call pargi (ndim)
		    call pargi (nline)
		call flush (STDOUT)
		}
	    im = ERR
	    return
	    }

	nord = IM_LEN(im,2)
	if (nline > nord) {
	    call eprintf ("GETTRES: Order %d > %d(2) in image %s\n")
		call pargi (nline)
		call pargi (nord)
		call pargstr (image)
	    call imunmap (im)
	    im = ERR
	    return
	    }

        ap = INDEFI

#  Set dispersion axis
	if (imaccf (im, "DISPAXIS") == NO) {
	    call imaddi (im,"DISPAXIS",1)
	    if (debug) {
		call eprintf ("GETTRES: Dispersion axis set to 1 in image %s\n")
		    call pargstr (image)
		}
	    }

#  Open spectrum world coordinate system
	if (debug) {
	    call printf ("GETTRES: Ready to open MWCS\n")
	    call flush (STDOUT)
	    }

	mw = smw_openim (im)
	if (mw == ERR) {
	    call eprintf ("GETTRES: MWCS error in image %s\n")
		call pargstr (image)
	    call imunmap (im)
	    im = ERR
	    return
	    }
	call smw_daxis (mw, im, 0, INDEFI, INDEFI)
	call smw_saxes (mw, NULL, im)

#  Open spectrum header
	if (debug) {
	    call printf ("GETTRES: Ready to open SH\n")
	    call flush (STDOUT)
	    }
	call shdr_open (im, mw, nline, nband, ap, SHDATA, sh)
	if (sh == ERR) {
	    call eprintf ("GETTRES: Spectrum header error in image %s\n")
		call pargstr (image)
	    call imunmap (im)
	    im = ERR
	    return
	    }

	if (strncmp (LABEL(sh),"Pix",3) == 0 || strncmp (LABEL(sh),"pix",3) == 0) {
	    if (debug) {
		call eprintf ("GETTRES: Non-spectral dispersion %s in image %s\n")
		    call pargstr (LABEL(sh))
		    call pargstr (image)
		}
	    call imunmap (im)
	    im = ERR
	    return
	    }

	npix = IM_LEN (im, 1)

#  Get spectrum order to use
	if (debug) call printf ("GETTRES: Ready to get image line\n")
	pix = SY(sh)

	if (debug) {
	    call printf ("GETTRES: line = %d, order = %d %d\n")
		call pargi (nline)
		call pargi (AP(sh))
		call pargi (ap)
	    }

#  Title is object name and order
	call strcpy (TITLE(sh), name, SZ_PATHNAME)
	ltitle = strlen (name)
	if (ltitle < 1) {
	    call strcpy (IM_TITLE(im), name, SZ_PATHNAME)
	    ltitle = strlen (name)
	    }
	if (ltitle < 1) {
	    ip = strldxs ("$/",image)
	    if (ip > 0)
		call strcpy (image[ip+1],name,SZ_PATHNAME)
	    else
		call strcpy (image, name, SZ_PATHNAME)
	    }
	call sprintf (temp, 16, "_%d")
	    call pargi (nline)
	call strcat (temp, name, SZ_PATHNAME)

	return
end


#  CLOSETRES -- Close IRAF image opened by GETTRES
 
procedure closetres (im, sh)
 
pointer	im		# Image header structure [returned NULL]
pointer	sh		# Spectrum header structure [returned NULL]

begin
#  Close spectrum WCS structure
	if (MW(sh) != NULL)
	    call smw_close (MW(sh))

#  Close spectrum header
	if (sh == ERR)
	    sh = NULL
	if (sh != NULL)
	    call shdr_close (sh)
	sh = NULL

#  Unmap the image
	if (im == ERR)
	    im = NULL
	if (im != NULL)
	    call imunmap (im)
	im = NULL

	return
end

# Jan 15 2009	New subroutines based on rvsao/Util/getimage.x and clsimage.x
# Mar  6 2009	Drop unused variables
